#!/usr/bin/env python

##TUI.py - A simple Python Textual User Interface library
##Copyright (C) 2004 Christopher Rebert
##
##This program is free software; you can redistribute it and/or
##modify it under the terms of the GNU General Public License
##as published by the Free Software Foundation; either version 2
##of the License, or (at your option) any later version.
##
##This program is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU General Public License for more details.
##
##You should have received a copy of the GNU General Public License
##along with this program; if not, write to the Free Software
##Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#Christopher Rebert, the author of TUI.py, can be contacted at the following email address:
#krebert AT mindspring DOT com
#Please send all bug reports, enhancement proposals, questions and comments to that address.

#VERSION: 1.3

#Planned for version 1.4: TextEditor - Help needed
#                         CellSelect
#                         Tree - Help Needed
#                         Tester

#changes made by Santosh Philip
#FileDirChooser() does not work correctly ... it has been updated

import os
from sys import stderr, platform
from os import listdir, mkdir
from os.path import exists
from time import sleep
from getpass import getpass, getuser
from dircache import annotate

__author__ = 'Christopher Rebert <krebert AT mindspring DOT com>'
__license__ = 'GNU General Public License'
__version__ = 1.3

def _PrintTitleLine(Title):
    """Prints title lines.
    
    Title - Title to print a line for.

    Intended for internal use only!"""
    
    print "="*(len(Title)+3)

def RadioButtons(Prompt, DescrObjs, Prefix=None, Mid=None):
    """Asks the user to choose one option from a group.
    
    Prompt - The string the user will be prompted with. A colon is always added to its end.
    DescrObjs - A tuple containing (Description,Object) pairs.
                Description is the string that the user will see as an option.
                Object is what will be returned if the user chooses its associated Description.
                The order is significant, as the options are presented in the order that they
                appear in DescrObjs.

    Optional:
        Prefix - The string to print along with a colon, a space, and then Mid, following the
                 title line. Both or neither must be given. If neither is given, they are not
                 printed. Default is None, meaning that Prefix and Mid won't be printed.
        Mid - See Prefix. Default is None, meaning that Mid and Prefix won't be printed."""
    
    ObjDict = {}
    Prompt += ":"
    if Mid and Prefix:
        Mid = Prefix+":"+Mid
    for Num in range(len(DescrObjs)):
        ObjDict[Num+1] = DescrObjs[Num]
    while True:
        print Prompt
        _PrintTitleLine(Prompt)
        if Mid and Prefix:
            print Mid
            print
        elif not Mid and Prefix:
            raise ValueError, "Both Prefix and Mid must be given, not just one of them"
        elif Mid and not Prefix:
            raise ValueError, "Both Prefix and Mid must be given, not just one of them"
        for Num in xrange(1, len(ObjDict)+1):
            print "["+`Num`+"]", ObjDict[Num][0]
        print
        try:
            Choice = ObjDict[int(raw_input("Enter the # of your choice: "))][1]
            print
            print
            break
        except (ValueError, KeyError, IndexError):
            print
            print
    return Choice

Menu = ListBox = Buttons = Tabs = DialogBox = RadioButtons

def CheckBoxes(Prompt, DescrObjs):
    """Asks the user to choose one or more options from a group.

    Prompt - The string the user will be prompted with. A colon is always added to its end.
    DescrObjs - A tuple containing (Description,Object) pairs.
                Description is the string that the user will see as an option.
                Object is what will be returned if the user chooses its associated Description.
                The order is significant, as the options are presented in the order that they
                appear in DescrObjs."""
    
    ObjDict = {}
    Prompt += ":"
    for Num in range(len(DescrObjs)):
        ObjDict[Num+1] = DescrObjs[Num]
    del DescrObjs
    while True:
        print Prompt
        _PrintTitleLine(Prompt)
        for Num in xrange(1, len(ObjDict)+1):
            print "["+`Num`+"]", ObjDict[Num][0]
        print
        CheckList = raw_input("Enter the #s of your choices separated by commas: ").split(",")
        print
        print
        try:
            CheckList = [ObjDict[int(Check)][1] for Check in CheckList]
            CheckList.sort()
            break
        except (KeyError, ValueError):
            continue
    return CheckList

CheckList = CheckBoxes

def ComboBox(Prompt, DescrObjs):
    """Asks the user to choose an option from a group or enter one of their own.

    Prompt - The string the user will be prompted with. A colon is always added to its end.
    DescrObjs - A tuple containing (Description,Object) pairs.
                Description is a string that the user will see as an option.
                Object is what will be returned if the user chooses its associated Description.
                The order is significant, as the options are presented in the order that they
                appear in DescrObjs."""
    
    ObjDict = {}
    Prompt += ":"
    for Num in range(len(DescrObjs)):
        ObjDict[Num+1] = DescrObjs[Num]
    Custom = len(ObjDict)+1
    ObjDict[Custom] = ("Type Your Own Option in",Custom)
    
    while True:
        print Prompt
        _PrintTitleLine(Prompt)
        for Num in xrange(1, len(ObjDict)+1):
            print "["+`Num`+"]", ObjDict[Num][0]
        print
        try:
            Choice = int(raw_input("Enter the # of your choice: "))

            if Choice == Custom:
                print
                print
                Choice = TextEntry(Prompt[:-1])
                for Val in ObjDict.values():
                    if Val[0] == Choice:
                        Choice = Val[1]
                        break
            else:
                Choice = ObjDict[Choice][1]
        
        except (ValueError, KeyError, IndexError):
            pass
        print
        print
    return Choice

def SpinBox(Prompt, Mode=0, Min=None, Max=None, Mul=None, NumObjs=None):
    """Asks the user for a number.

    Prompt - The string the user will be prompted with. A colon is always added to its end.

    Optional:
        Mode - Determines whether to return an int or a float, depending on what its type is.
               Default is integer.
        Min - The minimum number that the user will be allowed to enter. Default is None,
              meaning that there will be no minimum number.
        Max - The maximum number that the user will be allowed to enter. Default is None,
              meaning that there will be no maximum number.
        Mul - The number that the user enters must be a multiple of this for the number to be
              accepted. Default is None, meaning that there will be no such requirement.
        NumObjs - A tuple containing (Number,Object) pairs.
                  Object will be returned if the user chooses its associated Number. Default is
                  None, meaning that the user's number itself will be returned."""
    
    if not isinstance(Mode, int) and not isinstance(Mode, float):
        raise ValueError, "Mode should be an int or float not: "+type(Mode)
    Prompt += ":"
    while True:
        print Prompt
        _PrintTitleLine(Prompt)
        print "Min:",`Min`,
        print " Max:",`Max`,
        print " Multiple of:",`Mul`
        print
        try:
            Choice = raw_input("Enter a # that meets the requirements: ")
            if isinstance(Mode, int):
                Choice = int(Choice)
            elif isinstance(Mode, float):
                Choice = float(Choice)
        except ValueError:
            print
            print
            continue
        print
        print
        if Mul != None and Choice % Mul != 0:
            continue
        if Min == None and Max == None:
            break
        elif Min == None and New <= Max:
            break
        elif Max == None and Min <= New:
            break
        elif Min <= New <= Max:
            break
        else:
            continue
    try:
        Choice = NumObjs[Choice]
    except (KeyError, TypeError):
        pass
    return Choice

Scale = SpinBox

def TextEntry(Prompt, TextObjs=None):
    """Asks the user to enter text.

    Prompt - The string the user will be prompted with. A colon is always added to its end.

    Optional:
        TextObjs - A tuple containing (String,Object) pairs.
                   Object will be returned if the user inputs its associated String. Default is
                   None, meaning that the user's text itself will be returned."""

    
    Prompt += ":"
    print Prompt
    _PrintTitleLine(Prompt)
    Text = raw_input("Enter text: ")
    print
    print
    if TextObjs:
        try:
            Text = TextObjs[Text]
        except KeyError:
            pass
    return Text

TextField = TextEntry

def SecureTextEntry(Prompt, TextObjs):
    """Asks the user to enter text in a secure (if possible) way.

    Prompt - The string the user will be prompted with. A colon is always added to its end.

    Optional:
        TextObjs - A tuple containing (String,Object) pairs.
                   Object will be returned if the user inputs its associated String. Default is
                   None, meaning that the user's text itself will be returned."""
    
    Prompt += ":"
    print Prompt
    _PrintTitleLine(Prompt)
    print "Note: The text you enter may not be displayed at all"
    print
    Text = getpass(Prompt)
    print
    print
    if TextObjs:
        try:
            Text = TextObjs[Text]
        except KeyError:
            pass
    return Text

def UserPassPrompt(Prompt):
    """Securely asks the user for a username and password.

    Prompt - The string the user will be prompted with. A colon is always added to its end."""

    print Prompt
    _PrintTitleLine(Prompt)
    print "Note: Password may not be displayed at all"
    print
    User = raw_input("Username: ")
    Pass = getpass()
    print
    print
    return User, Pass

def PassPrompt(Prompt):
    """Securely asks the user for a password.

    Prompt - The string the user will be prompted with. A colon is always added to its end."""
    
    print Prompt
    _PrintTitleLine(Prompt)
    print "Note: Password may not be displayed at all"
    print
    Pass = getpass()
    print
    print
    return Pass

def PNDialogBox(Prompt, Type="oc", Default=True, PNObjs=None):
    """Asks the user a yes/no or ok/cancel question.

    Prompt - The string the user will be prompted with. A colon is always added to its end.

    Optional:
        Type - 'oc' for okay/cancel options, 'yn' for yes/no options. Default is 'oc'.
        Default - Indicates whether the default answer should be in the negative or affirmative
                  depending on whether Default is true or false. Default is true.
        PNObjs - A tuple containing (Truth,Object) pairs.
                 Object will be returned if the user selects the option implying the truth
                 value Truth. Default is None, meaning that the user's truth value itself will
                 be returned."""
    
    if Type == "oc":
        if Default:
            DescrObjs = (("Okay",True), ("Cancel",False))
        else:
            DescrObjs = (("Cancel",False), ("Okay",True))
    elif Type == "yn":
        if Default:
            DescrObjs = (("Yes",True), ("No",False))
        else:
            DescrObjs = (("No",False), ("Yes",True))
    else:
        raise ValueError, "Invalid value for Type and/or Default"
    
    Choice = RadioButtons(DescrObjs)
    if PNObjs:
        Choice = PNObjs[Choice]
    return Choice

def MessageBox(Title, Msg):
    """Displays a message.

    Title - The title of the message.
    Msg - The Message to be displayed."""
    
    print Title
    _PrintTitleLine(Title)
    print Msg, "\n"
    d = raw_input("Press Enter to Acknowledge")
    print
    print

def ErrorBox(Error):
    """Displays an error message.

    Error - The error to be displayed."""
    
    print >> stderr, "Error"
    print >> stderr, "========"
    print >> stderr, Error
    print >> stderr
    d = raw_input("Press Enter to Acknowledge")
    print
    print

def AboutBox(AppName, Description, Year, Holder, Credits):
    """Displays general program information.

    AppName - The name of the application.
    Description - A description of the application.
    Year - The year(s) the application was copywritten.
    Holder - The application's copyright owner(s).
    Credits - The credits for the application."""

    if isinstance(Year, str):
        YearLen = len(Year)
    elif isinstance(Year, int):
        YearLen = len(`Year`)
    
    if len(AppName)+len(Description)+6 > YearLen+len(Holder)+15:
        TitleLineLen = len(AppName)+len(Description)+6
    else:
        TitleLineLen = YearLen+len(Holder)+18
    
    print AppName,"-",Description
    sleep(0.5)
    print "Copyright (C)", Year, Holder
    sleep(0.5)
    _PrintTitleLine(" "*TitleLineLen)
    sleep(0.5)
    for Line in Credits.split("\n"):
        print Line
        sleep(0.5)
    print
    d = raw_input("Press Enter to Acknowledge")
    print
    print

Credits = AboutBox

def Tooltip(Tip):
    """Displays a tooltip.

    Tip - The tooltip to display."""
    
    print Tip
    sleep(3)
    print
    print

Balloon = Tooltip

def ProgressBar(Title, Percent, InProgress=None):
    """Displays a progress bar.

    Title - Title of the progress bar. Three periods are automatically added to the end.
    Percent - A number 1-100 indicating what percent of the task is done.

    Optional:
        InProgress - What part of the task is currently being performed. Three periods are
                     automatically added to the end.
    """
    
    if 0 > Percent > 100:
        raise ValueError, "Percent must be between 1-100, not "+`Percent`
    Title += "..."
    print Title
    _PrintTitleLine(Title)
    _ProgressBar(Percent)
    if InProgress:
        print "Status:", InProgress+"..."
    print
    print

Meter = ProgressBar
    
def _ProgressBar(Percent):
    """Displays an individual progress bar.

    Percent - Percent to fill the progress bar to.

    Intended for internal use only."""
    
    Bars = int(round(Percent))//5
    print "["+"*"*Bars+(20-Bars)*" "+"]",`Percent`+"%",
    if Bars == 20:
        print "Done"
    else:
        print

def Bell(Prompt):
    """Displays a message to get the user's attention.

    Prompt - The prompt to alert the user with."""
    
    Border = int(round(len(Prompt)/2.0))*"*"
    HeadLine = Border+"ATTENTION"+Border
    ScrollingText(5*(HeadLine+"\n"+Prompt+"\n"))
    d = raw_input("Press Enter to Acknowledge")
    print
    print

Alert = FlashTitleBar = Bell

def ScrollingText(Text):
    """Displays some text line by line (marquee-style). (User cannot scroll)

    Text - The text to display."""
    
    for Line in [Item for Item in Text.split("\n") if Item]:
        print Line
        sleep(0.5)
    print

Ticker = ScrollingText

def ScrolledText(Prompt, Text, Lines=5):
    """Displays text that the user can scroll through. (The text cannot be edited)

    Prompt - The string the user will be prompted with. A colon is always added to its end.
    Text - The text to be displayed.

    Optional:
        Lines - The number of lines displayed per screen. Default is 5."""

    Prompt += ":"
    LineList = Text.split("\n")
    Start = 0
    if Start+Lines > len(LineList):
        Lines = len(LineList)
    
    while True:
        print Prompt
        _PrintTitleLine(Prompt)
        
        if Start > len(LineList):
            Start = Start-len(LineList)-1
        elif Start < 0:
            Start = 0
        
        if len(LineList) > Start+Lines:
            for Slice in range(Start, Start+Lines):
                print LineList[Slice]
        else:
            for Slice in range(Start,len(LineList)):
                print LineList[Slice]

        print
        d = raw_input("Press Enter to Acknowledge")
        print
        print
        Start = RadioButtons("Choose an action",( ("Scroll Up One Screen",Start-Lines),
                                                  ("Scroll Down One Screen",Start+Lines),
                                                  ("Scroll Up One Line", Start-1),
                                                  ("Scroll Down One Line",Start+1),
                                                  ("Scroll to a Line by Line #", "n"),
                                                  ("Stop Viewing Text","e") ))
        if Start == "e":
            break
        elif Start == "n":
            Start = SpinBox("Enter line #", 1, len(LineList))-1
        if Start > len(LineList)-1:
            Start = 0
            continue
        elif Start < 0:
            Start = len(LineList)-1
            continue

def FileDirEntry(Prompt, Mode="f", Exts=()):
    """Asks the user to enter the path to a file or directory.

    Prompt - The string the user will be prompted with. A colon is always added to its end.

    Optional:
        Mode - 'f' or 'd' depending on whether the user is to select a file or directory.
               Default is 'f' for files.
        Exts - A tuple containing the extensions (without periods) of the types of files the
               user will be allowed to select. Default is an empty tuple for any extension."""
    
    if Mode != "f" and Mode != "d":
        raise ValueError, "Mode must be 'f' or 'd', not "+`Mode`
    Prompt += ":"
    while True:
        print Prompt
        _PrintTitleLine(Prompt)
        
        if Mode == "f":
            Prompt_ = "Enter the path to a file: "
        elif Mode == "d":
            Prompt_ = "Enter the path to a directory: "
        FileDir = raw_input(Prompt_)
        FileDir = FileDir.replace("\\","/")
        printEnvironmentError
        print
        if exists(FileDir):
            if Mode == "f" and FileDir[-1] != "/" and _FileDirNameChk(FileDir, Exts):
                try:
                    d = open(FileDir,"r")
                    d.close()
                    break
                except EnvironmentError:
                    continue
            elif Mode == "d" and FileDir[-1] == "/":
                try:
                    d = open(FileDir,"r")
                    d.close()
                    continue
                except EnvironmentError:
                    break
    return FileDir

def FileDirChooser(Prompt, Mode="f", Exts=(),startdir=os.getcwd()+'/'):
    """Asks the user to browse for a file or directory.

    Prompt - The string the user will be prompted with. A colon is always added to its end.

    Optional:
        Mode - 'f' or 'd' depending on whether the user is to select a file or directory.
               Default is 'f' for files.
        Exts - A tuple containing the extensions (without periods) of the types of files the
               user will be able to select. Default is an empty tuple for any extension."""
    
    if Mode != "f" and Mode != "d":
        raise ValueError, "Mode must be 'f' or 'd', not: "+`Mode`
    if platform.count("unix") or platform.count("linux") or platform.count("gnu")\
       or platform.count("sun"):
        Dir = startdir#~" Check if "~" works on unix
    else:
        Dir = startdir
    while True:
        Dir = Dir.replace("\\","/")
        Contents = listdir(Dir)
        annotate(Dir,Contents)
        Contents.sort()
        DescrObjs = [(Item,Item) for Item in Contents if _FileDirNameChk(Item, Exts)]
        if Dir != "/":
            DescrObjs.append(("Go Up One Directory Level",1))
        DescrObjs.append(("Create a New Subdirectory",2))
        if platform == "win32":
            DescrObjs.append(("Go to Your Desktop",3))
            DescrObjs.append(("Go to Shared Desktop",4))
            DescrObjs.append(("Go to My Documents",5))
            DescrObjs.append(("Go to C:\\",6))
            DescrObjs.append(("Go to A:\\",7))
            DescrObjs.append(("Go to Another Drive",8))
        DescrObjs.append(("Cancel",9))
        FileDir = RadioButtons(Prompt, DescrObjs, "Current Directory", Dir)
        if FileDir == 9:
            FileDir = -1
            break
        elif FileDir == 1:
            Dir = "/".join( Dir.split("/")[:-2] )+"/"
            continue
        elif FileDir == 2:
            NewDir = Dir+TextEntry("Enter a name for the new directory")+"/"
            try:
                mkdir(NewDir)
                Dir = NewDir
            except EnvironmentError:
                pass
            continue
        elif FileDir == 3:
            Dir = "/Documents and Settings/"+getuser()+"/Desktop/"
            continue
        elif FileDir == 4:
            Dir = "/Documents and Settings/All Users/Desktop/"
            continue
        elif FileDir == 5:
            Dir = "/Documents and Settings/"+getuser()+"/My Documents/"
            continue
        elif FileDir == 6:
            Dir = "C:/"
            continue
        elif FileDir == 7:
            try:
                d = listdir("A:/")
                Dir = "A:/"
            except EnvironmentError:
                pass
            continue
        elif FileDir == 8:
            NewDir = TextEntry("Enter the drive letter(s)")+":/"
            try:
                if exists(NewDir):
                    Dir = NewDir
            except WindowsError:
                pass
            continue
        elif Mode == "d" and FileDir[-1] == "/":
            Action = RadioButtons("Choose an action",( ("Browse in this Directory",1),("Select this Directory",2) ))
            if Action == 1:
                Dir = Dir+FileDir
                continue
            elif Action == 2:
                break
        elif FileDir[-1] == "/":
            Dir = Dir+FileDir
            continue
        else:
            break
    if FileDir==-1: return FileDir
    return Dir+FileDir

def _FileDirNameChk(Name, Exts):
    """Checks if a file ends in a certain extension.

    Name - Name of the file to check
    Exts - A tuple containing the allowed extensions. (Without periods)

    Intended for internal use only"""

    Status = False
    if Name[-1] == "/":
        return True
    if not Exts:
        return True
    for Extension in Exts:
        try:
            Extension = "."+Extension
            if Name[-len(Extension):] == Extension:
                Status = True
                break
        except IndexError:
            pass
    return Status

def Tips(AppName, SubtitleTips):
    """Lets the user browse through software tips.

    AppName - The name of the application to display tips for.
    SubtitleTips - A tuple containg other tuples of (Subtitle,Tip).
                   Subtitle - The title of the tip.
                   Tip - The tip itself."""
    
    Position = 0
    while True:
        print AppName, "Tips"
        _PrintTitleLine(AppName+" "*5)
        print SubtitleTips[Position][0]
        print
        print SubtitleTips[Position][1]
        print
        d = raw_input("Press Enter to Acknowledge")
        print
        print
        Action = RadioButtons("Choose an action",( ("Next Tip",1), ("Previous Tip",-1),
                                                   ("Exit",0) ))
        if Action == 1:
            Position += 1
        elif Action == -1:
            Position -= 1
        elif Action == 0:
            break
        if Position > len(SubtitleTips)-1:
            Position = 0
        elif Position < 0:
            Position = len(SubtitleTips)-1

def Help(AppName, TopicDocs, TermDefs):
    """Provides a simple user help system.
    
    AppName - Name of the application to give help for.
    TopicDocs - A tuple containing pairs of (Topic,Docs).
                Topic - A help topic.
                Docs - Documentation on Topic's topic.
    TermDefs - A dictionary of Term:Def.
               Term - The term.
               Def - Term's definition."""
    
    while True:
        Section = RadioButtons(AppName+" Help System",( ("Contents",1),("Index",2),("Search",3),
                                              ("Exit Help",-1) ))
        if Section == 1:
            _HelpContents(AppName, TopicDocs)
        elif Section == 2:
            _HelpIndex(AppName, TermDefs)
        elif Section == 3:
            _HelpSearch(AppName, TermDefs)
        elif Section == -1:
            break
        
def _HelpContents(AppName, TopicDocs):
    """A simple help contents.

    AppName - The name of the application to display a help contents for.
    TopicDocs - A tuple containing pairs of (Topic,Docs).
                Topic - A help topic.
                Docs - Documentation on Topic's topic.
    
    Intended for internal use only."""
    
    TopicDocs = [(Entry[0],(Entry[0], Entry[1])) for Entry in TopicDocs]
    TopicDocs.append(("Return to Help Menu",-1))
    while True:
        Docs = RadioButtons(AppName+" Help Contents - Choose a topic", TopicDocs)
        if Docs == -1:
            break
        print AppName+" Help Contents"
        _PrintTitleLine(AppName+" "*14)
        print Docs[0]
        print
        print Docs[1]
        print
        d = raw_input("Press Enter to Acknowledge")
        print
        print

def _HelpIndex(AppName, TermDefs):
    """A simple help index.

    AppName - The name of the application to display a help index for.

    TermDefs - A dictionary of Term:Def.
               Term - The term.
               Def - Term's definition.
    
    Intended for internal use only"""
    
    while True:
        IndexView = [(Key,(Key, TermDefs[Key])) for Key in TermDefs]
        IndexView.append(("Return to Help Menu",-1))
        WordDef = RadioButtons(AppName+" Help Index - Choose a term", IndexView)
        if WordDef == -1:
            break
        print AppName+" Help Index"
        _PrintTitleLine(AppName+" "*11)
        print WordDef[0]
        print
        print WordDef[1]
        print
        d = raw_input("Press Enter to Acknowledge")
        print
        print

def _HelpSearch(AppName, SearchDict):
    """A simple help search.

    AppName - The name of the application to display a help index for.
    TermDefs - A dictionary of Term:Def.
               Term - The term.
               Def - Term's definition.
    
    Intended for internal use only."""
    
    while True:
        Action = RadioButtons(AppName+" Help Search - Choose an action",(
            ("Search for (a) word(s)",1),("Return to Help Menu",-1) ))
        if Action == -1:
            break
        elif Action == 1:
            Words = TextEntry("Enter search terms")
            print "Now Searching, Please Wait..."
            WordList = Words.split(" ")
            WordList = [Item for Item in WordList if Item != ""]
            Results = []
            for Word in WordList:
                Word = Word.lower()
                for Key in SearchDict.keys():
                    Key = Key.lower()
                    if Key.count(Word):
                        Results.append((Key, (Key,SearchDict[Key])))
            print
            print
            if not len(Results):
                print "Sorry, no search keys matched your terms."
                print "Try again using different words, or consult the Help Index"
                print
                d = raw_input("Press Enter to Acknowledge")
                print
                print
                continue
            TitleDoc = RadioButtons(AppName+" Help Search - Choose a search result", Results,
                                    "Search query", Words)
            print AppName+" Help Search - Result"
            _PrintTitleLine(AppName+" "*21)
            print TitleDoc[0]
            print
            print TitleDoc[1]
            print
            d = raw_input("Press Enter to Continue")
            print
            print
            continue

def Tree(Prompt, TreeLeaves):#Needs to be developed
    """Placeholder for a future tree selector."""

def TextEditor(Prompt):#Needs to be developed
    """Placeholder for a future text editor."""

def CellSelect(Prompt, Table):#Needs to be developed
    """Placeholder for a future table selection."""

if __name__ == "__main__":
    Creds = """Programmed by: Christopher Rebert
Special Thanks: GvR
                PSF
                Python Developers
                IDLE Developers
                Thomas Churm

    PYTHON POWERED!!!

Thanks for using TUI.py v1.3
Check our listing on PyPI for updates."""
    AboutBox("TUI.py", "A simple Python Textual User Interface library", 2004,
             "Christopher Rebert", Creds)



#!/usr/bin/env python

## EPlusInterface (EPI) - An interface for EnergyPlus
## Copyright (C) 2004 Santosh Philip
##
## This file is part of EPlusInterface.
## 
## EPlusInterface is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## EPlusInterface is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with EPlusInterface; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
##
##
## Santosh Philip, the author of EPlusInterface, can be contacted at the following email address:
## santosh_philip AT yahoo DOT com
## Please send all bug reports, enhancement proposals, questions and comments to that address.
## 
## VERSION: 0.004

#open any file VERSION: 0.002
#edit entry
#using cPickle


import sys
import getopt
import os
import string
import user
import copy

import eplusdata
import parse_idd
import slashcomment

__author__ = 'Santosh Philip <santosh_philip AT Yahoo DOT com>'
__license__ = 'GNU General Public License'
__version__ = '0.003'


sys.path.append(user.home+'/Library/PythonFiles')
import mylib1,mylib2,mylib3
import tui1

def objselection(obj):
	"""
	used where a field refers to an object-list
	This routine allows you to select an item from the list of references
	"""
	global data
	global block
	global commdct
	global objlst_dct
	ls=data.getrefs(objlst_dct[obj])
	DescrObjs=[]
	for el in ls:
		DescrObjs.append((el,el))
	
	Prompt='Select an Item'
	return tui1.RadioButtons(Prompt,DescrObjs)
		

def keyselection(thekeys):
	"""
	There is a choice of values that can fill this field
	keyselection(thekeys) allows you to make the selection
	"""
	DescrObjs=[]
	for el in thekeys['key']:
		DescrObjs.append((el,el))
	
	Prompt='hit a button'
	return tui1.RadioButtons(Prompt,DescrObjs)

def tuisavefile(filename):
	"""
	Save the Energyplus file
	"""
	global data
	Prompt='Would you like to save this file ?'
	Action = tui1.RadioButtons("Choose an action",( ("Save",1),("Save As",2),("Quit",3) ))
	if Action==3:
		return
	if Action==1:
		mylib1.writeStr2File(filename,`data`)
		return
	Prompt='Choose new file name:'
	filename=tui1.TextEntry(Prompt)
	mylib1.writeStr2File(filename,`data`)
	
def tuiopenfile():
	"""
	Open an Energyplus file
	"""
	Prompt='make a selection to open a file'
	result=tui1.FileDirChooser(Prompt,'f',('idf',))
	return result

def allobjects():
	"""
	Show all objects in the EnergyPlus file
	print the object type
	"""
	global data
	global block
	global commlst
	global commdct
	fullls=[]
	for el in data.dtls:
		if data.dt[el]!=[]:
			fullls.append(el)
	DescrObjs=[]
	for el in fullls:DescrObjs.append((el,el))
	DescrObjs.append(('-- Quit --','#up#'))
	
	Prompt='hit a button'
	stay=1
	while stay:
		sel=tui1.RadioButtons(Prompt,DescrObjs)
		if sel=='#up#':return
		thisobject(sel)
	#==================================

def thisobject(sel1):	
	"""
	Show all the instances of this object
	if it is an unique object (only one instance possible)
	then goto allfields
	"""
	global data
	global block
	global commlst
	global commdct
	seldt=data.dt[sel1]
	
	Prompt='hit a button for:'+sel1
	stay=1
	index=slashcomment.getobjectindex(block,sel1)
	a=slashcomment.slashcomment()
	keyshere=a.keyshere(commdct[index][0])
	while stay:
		if 'unique-object' in keyshere:
			allfields(sel1,0)
			return
		DescrObjs=[]
		for i in range(0,len(seldt)):
			DescrObjs.append((seldt[i][1],i))
		DescrObjs.append(('-- Go up in menu --','#up#'))
		sel2=tui1.RadioButtons(Prompt,DescrObjs)
		if sel2=='#up#':return
		allfields(sel1,sel2)
	#==================================

def allfields(sel1,sel2):
	"""
	Show all fields of this object
	"""
	global data
	global block
	global commlst
	global commdct
	seldt=data.dt[sel1]
	blocknames=[]
	for el in data.dtls:
		blocknames.append(el)
	
	name=data.dt[sel1][sel2][0]
	print 'finding  '+name.upper()
	i=blocknames.index(name.upper())
	comm=commlst[i]
	

	Prompt='hit a button :' 
	stay=1
	while stay:
		item=data.dt[sel1][sel2]
		DescrObjs=[]
		for i in range(0,len(item)):
			if len(comm[i])==0:
				comment=''
			else:
				comment=comm[i][0]
			DescrObjs.append((item[i] + ' => ' + comment,i))
		DescrObjs.append(('-- Go up in menu --','#up#'))
		sel3=tui1.RadioButtons(Prompt,DescrObjs)
		if sel3=='#up#':return
		thisfield(comm,sel1,sel2,sel3)
	#==================================

def thisfield(comm,sel1,sel2,sel3):
	"""
	Show this field
	with Slash comments
	"""
	global dirty
	global data
	global block
	global commdct
	keeplooping=True
	while keeplooping:
		seldt=data.dt[sel1]
		item=data.dt[sel1][sel2][sel3]
		print item
		print '-'*len(item)
		print 'This is:'
		for el in comm[sel3]:
			print '\t'+el
		DescrObjs=[]
		DescrObjs.append(('Edit this value','#edit#'))
		DescrObjs.append(('-- Go up in menu --','#up#'))
		Prompt='hit a button :' 
		sel4=tui1.RadioButtons(Prompt,DescrObjs)
		if sel4=='#up#':return
		if sel4=='#edit#':
			keeplooping=editfield(sel1,sel2,sel3)
			
	#==================================

def editfield(sel1,sel2,sel3):
	"""
	Edit this field
	different types of field entries(key, object-list, real) 
	will result in diffferent interactions.
	"""
	global dirty
	global data
	global block
	global commdct
	print sel1
	index=slashcomment.getobjectindex(block,sel1)
	#-----key-----
	a=slashcomment.slashcomment()
	keyshere=a.keyshere(commdct[index][sel3])
	if 'key' in keyshere:
		#go to keyselection routine
		val=keyselection(commdct[index][sel3])
	elif 'object-list' in keyshere:
		val=objselection(commdct[index][sel3]['object-list'][0])
	elif 'autosizable' in keyshere:
		# button for 1.autosize and 2.enter number
		DescrObjs=[]
		DescrObjs.append(('autosize','autosize'))
		DescrObjs.append(('Enter a number','#EnterNumber#'))
		Prompt='Select an Option :' 
		autosizeselection=tui1.RadioButtons(Prompt,DescrObjs)
		if autosizeselection=='autosize':
			val=autosizeselection
		else:
			Prompt='Enter new Value'
			val=tui1.TextEntry(Prompt)		
	else:
		Prompt='Enter new Value'
		val=tui1.TextEntry(Prompt)
	try:
		slashcomment.checkVal(val,commdct[index][sel3])
		data.dt[sel1][sel2][sel3]=val
		dirty=True
		keeplooping=False
		return keeplooping
	except "anError",descr:
		print descr
		tmp=raw_input('hit RETURN to continue')
		keeplooping=True
		return keeplooping


def showlicense():
	"""
	print the preamble to the GPL license
	This function is called when using TUI as the interface
	"""
	print """

EPlusInterface (EPI) - An interface for EnergyPlus
Copyright (C) 2004 Santosh Philip

EPlusInterface is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

EPlusInterface is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EPlusInterface; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 


Santosh Philip, the author of EPlusInterface, can be contacted at the following email address:
santosh_philip AT yahoo DOT com
Please send all bug reports, enhancement proposals, questions and comments to that address.

VERSION: 0.003

	"""
	print '='*35

def main():
	"""
	da main
	"""
	global data
	global block
	global commlst
	global commdct
	global objlst_dct
	showlicense()
	print 'It is a little slow here parsing data from the Energy.idd file'
	print 'Please wait .... '
	dictfile=fileloc+'Energy+V6_0.idd'
	
	block,commlst,commdct=parse_idd.extractidddata(dictfile)
	objlst_dct=parse_idd.getobjectref(block,commdct)
	tmp=raw_input('hit RETURN to continue')

	fname1=tuiopenfile()
	if fname1==-1:return

	
	theidd=eplusdata.idd(block,2)
	data=eplusdata.eplusdata(theidd,fname1)
		
	allobjects()
	global dirty
	if dirty:
		#save the file
		tuisavefile(fname1)
		#mylib1.writeStr2File('saved.idf',`data`)
		
#global variables
fileloc='../../../iddfiles/'
global dirty
global data
global block
global commlst
global commdct
global objlst_dct
dirty=False
if __name__ == "__main__":
    main()



# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

"""just read the idf file"""

import sys
import getopt
import parse_idd
import eplusdata
# from EPlusInterfaceFunctions import parse_idd
# from EPlusInterfaceFunctions import eplusdata

def readidf(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block,commlst,commdct=parse_idd.extractidddata(iddfile)

    theidd=eplusdata.idd(block,2)
    data = eplusdata.eplusdata(theidd,idfname)
    return data


def readiddidf(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block,commlst,commdct=parse_idd.extractidddata(iddfile)

    theidd=eplusdata.idd(block,2)
    data = eplusdata.eplusdata(theidd,idfname)
    return theidd, data

def readiddstuff(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block,commlst,commdct=parse_idd.extractidddata(iddfile)

    theidd=eplusdata.idd(block,2)
    data = eplusdata.eplusdata(theidd,idfname)
    return block,commlst,commdct
    
    

def readdatacommlst(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block,commlst,commdct=parse_idd.extractidddata(iddfile)

    theidd=eplusdata.idd(block,2)
    data = eplusdata.eplusdata(theidd,idfname)
    return data, commlst
    
def readdatacommdct(idfname, iddfile='Energy+.idd', commdct=None):
    """read the idf file"""
    if not commdct:
        block,commlst,commdct=parse_idd.extractidddata(iddfile)
        theidd=eplusdata.idd(block,2)
    else:
        theidd = iddfile
    data = eplusdata.eplusdata(theidd,idfname)
    return data, commdct
    
def readdatacommdct1(idfname, iddfile='Energy+.idd',
                                    commdct=None, block=None):
    """read the idf file"""
    if not commdct:
        block,commlst,commdct=parse_idd.extractidddata(iddfile)
        theidd=eplusdata.idd(block,2)
    else:
        theidd = eplusdata.idd(block,2)
    data = eplusdata.eplusdata(theidd,idfname)
    return block, data, commdct



# EPlusInterface (EPI) - An interface for EnergyPlus
# Copyright (C) 2004 Santosh Philip

# This file is part of EPlusInterface.
# 
# EPlusInterface is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# EPlusInterface is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with EPlusInterface; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 


# Santosh Philip, the author of EPlusInterface, can be contacted at the following email address:
# santosh_philip AT yahoo DOT com
# Please send all bug reports, enhancement proposals, questions and comments to that address.
# 
# VERSION: 0.004
#last update 21 Apr 2004


#this is a test version ... not for real use
#dammit i am using it

import sys
import getopt
import os
import string
import user
import copy



sys.path.append(user.home+'/Library/PythonFiles')
import mylib1,mylib2,mylib3



def removecomment(st,c):
    # the comment is similar to that in python.
    # any charachter after the # is treated as a comment 
    # until the end of the line
    # st is the string to be de-commented
    # c is the comment phrase
    linesep=mylib3.getlinesep(st)
    ls=st.split(linesep)
    for i in range(len(ls)):
        l=ls[i].split(c)
        ls[i]=l[0]
    
    return string.join(ls,linesep)
    

class idd:
    def __init__(self,dictfile,version=2):
        if version==2:
            # version==2. This is a just a flag I am using
            # it may wind up being the only type... then I can clean this up 
            # and not use the other option
            self.dt,self.dtls=self.initdict2(dictfile)
            return
        self.dt,self.dtls=self.initdict(dictfile)
        
    def initdict2(self,dictfile):           
        dt={}
        dtls=[]
        d=dictfile
        for el in d:
            dt[el[0].upper()]=[] #dict keys for objects always in caps
            dtls.append(el[0].upper())
        return dt,dtls

    def initdict(self,fname):
        st=mylib2.readfile(fname)
        nocom=removecomment(st,'!')
        idfst=nocom
        ls=string.split(idfst,';')
        lss=[]
        for el in ls:
            lst=string.split(el,',')
            lss.append(lst)
            
        for i in range(0,len(lss)):
            for j in range(0,len(lss[i])):
                lss[i][j]=lss[i][j].strip()
            
        dt={}
        dtls=[]
        for el in lss:
            if el[0]=='':continue
            dt[el[0].upper()]=[]
            dtls.append(el[0].upper())
            
        return dt,dtls



class eplusdata:

    def __init__(self,dictfile=None,fname=None):
        if fname==None and dictfile==None:
            self.dt,self.dtls={},[]
        if isinstance(dictfile,str) and fname==None:
            self.initdict(dictfile)
        if isinstance(dictfile,idd) and fname==None:
            self.initdict(dictfile)
        if isinstance(fname,str) and isinstance(dictfile,str):
            fnamefobject = open(fname, 'r')
            self.makedict(dictfile,fnamefobject)
        if isinstance(fname,str) and isinstance(dictfile,idd):
            fnamefobject = open(fname, 'r')
            self.makedict(dictfile,fnamefobject)
        from StringIO import StringIO
        if isinstance(fname,(file, StringIO)) and isinstance(dictfile,str):
            self.makedict(dictfile,fname)
        if isinstance(fname,(file, StringIO)) and isinstance(dictfile,idd):
            self.makedict(dictfile,fname)

    def __repr__(self):
        #print dictionary
        dt=self.dt
        dtls=self.dtls
        dossep=mylib3.unixsep # using a unix EOL
        st=''
        for node in dtls:
            nodedata=dt[node.upper()]
            for block in nodedata:
                for i in range(len(block)):
                    format='     %s,'+dossep
                    if i==0:format='%s,'+dossep
                    if i==len(block)-1:format='     %s;'+dossep*2
                    st=st+ format %block[i]
        
        return st

    #------------------------------------------
    def initdict(self,fname):
        #create a blank dictionary
        if isinstance(fname,idd):
            self.dt,self.dtls=fname.dt,fname.dtls
            return self.dt,self.dtls
        
        st=mylib2.readfile(fname)
        nocom=removecomment(st,'!')
        idfst=nocom
        ls=string.split(idfst,';')
        lss=[]
        for el in ls:
            lst=string.split(el,',')
            lss.append(lst)
        
        for i in range(0,len(lss)):
            for j in range(0,len(lss[i])):
                lss[i][j]=lss[i][j].strip()
        
        dt={}
        dtls=[]
        for el in lss:
            if el[0]=='':continue
            dt[el[0].upper()]=[]
            dtls.append(el[0].upper())
        
        self.dt,self.dtls=dt,dtls
        return dt,dtls
    
    #------------------------------------------
    def makedict(self,dictfile,fnamefobject):
        #stuff file data into the blank dictionary
        #fname='./exapmlefiles/5ZoneDD.idf'
        #fname='./1ZoneUncontrolled.idf'
        if isinstance(dictfile,idd):
            localidd=copy.deepcopy(dictfile)
            dt,dtls=localidd.dt,localidd.dtls
        else:
            dt,dtls=self.initdict(dictfile)
        # st=mylib2.readfile(fname)
        st = fnamefobject.read()
        fnamefobject.close()
        nocom=removecomment(st,'!')
        idfst=nocom
        ls=string.split(idfst,';')
        lss=[]
        for el in ls:
            lst=string.split(el,',')
            lss.append(lst)
        
        for i in range(0,len(lss)):
            for j in range(0,len(lss[i])):
                lss[i][j]=lss[i][j].strip()
        
        for el in lss:
            node=el[0].upper()
            if dt.has_key(node):
                #stuff data in this key
                dt[node.upper()].append(el)
            else:
                #scream
                if node=='':continue
                print 'this node -%s-is not present in base dictionary'%(node)
            
        self.dt,self.dtls=dt,dtls
        return dt,dtls
    
    def replacenode(self,othereplus,node):
        #replace the node here with the node from othereplus
        node=node.upper()
        self.dt[node.upper()]=othereplus.dt[node.upper()]
    
    def add2node(self,othereplus,node):
        #add the node here with the node from othereplus
        #this will potentially have duplicates
        node=node.upper()
        self.dt[node.upper()]=self.dt[node.upper()] + othereplus.dt[node.upper()]
    
    def addinnode(self, otherplus, node, objectname):
       """add an item to the node. 
       example: add a new zone to the element 'ZONE' """
       # do a test for unique object here
       newelement = otherplus.dt[node.upper()]
       
    def getrefs(self,reflist):
        """
        reflist is got from getobjectref in parse_idd.py
        getobjectref returns a dictionary. 
        reflist is an item in the dictionary
        getrefs gathers all the fields refered by reflist
        """
        ls=[]
        for el in reflist:
            if self.dt.has_key(el[0].upper()):
                for elm in self.dt[el[0].upper()]:
                    ls.append(elm[el[1]])
        return ls

    
#------------------------------------------




# Copyright (c) 2012 Tuan Tran

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

""""This module is used for calculation of zone area for E+ surfaces"""
# Wrote by Tuan Tran trantuan@hawaii.edu / tranhuuanhtuan@gmail.com
# School of Architecture, University of Hawaii at Manoa

# import area_surface as surf
import surface 

def area(poly):
    """Calculation of zone area"""
    poly_xy = []
    N = len(poly)
    for i in range(N):
        poly[i] = poly[i][0:2] + (0,)
        poly_xy.append(poly[i])
    return surface.area(poly)


# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

"""use epbunch"""

from EPlusInterfaceFunctions import readidf
import bunchhelpers
from bunch_subclass import EpBunch
from bunch_subclass import fieldnames, fieldvalues, GetRange, CheckRange
import iddgaps
import function_helpers as fh

def iddversiontuple(afile):
    """given the idd file or filehandle, return the version handle"""
    def versiontuple(v):
        return tuple(map(int, (v.split("."))))
    if type(afile) == str:
        fhandle = open(afile, 'r')
    else:
        fhandle = afile
    line1 = fhandle.readline()
    line = line1.strip()
    if line1 == '':
        return (0, )
    v = line.split()[-1]
    return versiontuple(v)
    

def makeabunch(commdct, obj, obj_i):
    """make a bunch from the object"""
    objidd = commdct[obj_i]
    objfields = [comm.get('field') for comm in commdct[obj_i]]
    objfields[0] = ['key']
    objfields = [field[0] for field in objfields]
    obj_fields = [bunchhelpers.makefieldname(field) for field in objfields]
    bobj = EpBunch(obj, obj_fields, objidd)
    return bobj

def makebunches(data, commdct):
    """make bunches with data"""
    bunchdt = {}
    dt, dtls = data.dt, data.dtls
    for obj_i, key in enumerate(dtls):
        key = key.upper()
        bunchdt[key] = []
        objs = dt[key]
        for obj in objs:
            # if obj[0] == "Construction:WindowDataFile":
            #     print obj
            bobj = makeabunch(commdct, obj, obj_i)
            bunchdt[key].append(bobj)
    return bunchdt

def convertfields(key_comm, obj):
    """convert the float and interger fields"""
    def apass(a):
        return a
    typefunc = dict(integer=int, real=float)
    # types = [comm.get('type', [None])[0] for comm in key_comm]
    types = []
    for comm in key_comm:
        types.append(comm.get('type', [None])[0])
    convs = [typefunc.get(typ, apass) for typ in types]
    for i, (val, conv) in enumerate(zip(obj, convs)):
        try:
            val = conv(val)
            obj[i] = val
        except ValueError, e:
            pass
    return obj
    
def convertallfields(data, commdct):
    """docstring for convertallfields"""
    for key in data.dt.keys():
        objs = data.dt[key]
        for i, obj in enumerate(objs):
            key_i = data.dtls.index(key)
            key_comm = commdct[key_i]
            obj = convertfields(key_comm, obj)
            objs[i] = obj
            
def addfunctions(dtls, bunchdt):
    """add functions to the objects"""
    snames = ["BuildingSurface:Detailed",
    "Wall:Detailed",
    "RoofCeiling:Detailed",
    "Floor:Detailed",
    "FenestrationSurface:Detailed",
    "Shading:Site:Detailed",
    "Shading:Building:Detailed",
    "Shading:Zone:Detailed",]
    for sname in snames:
        if bunchdt.has_key(sname.upper()):
            surfaces = bunchdt[sname.upper()]
            for surface in surfaces:
                surface.__functions = {'area':fh.area,
                    'height':fh.height, # not working correctly
                    'width':fh.width, # not working correctly
                    'azimuth':fh.azimuth,
                    'tilt':fh.tilt,
                    'coords':fh.getcoords, # needed for debugging
                    } 
    # add common functions
    # for name in dtls:
    #     for idfobject in bunchdt[name]:
    #         # idfobject.__functions 
    #         idfobject['__functions']['fieldnames'] = fieldnames
    #         idfobject['__functions']['fieldvalues'] = fieldvalues
    #         idfobject['__functions']['getrange'] = GetRange(idfobject)
    #         idfobject['__functions']['checkrange'] = CheckRange(idfobject)
        
            

def idfreader(fname, iddfile, conv=True):
    """read idf file and reutrn bunches"""
    data, commdct = readidf.readdatacommdct(fname, iddfile=iddfile)
    if conv:
        convertallfields(data, commdct)
    # fill gaps in idd
    dt, dtls = data.dt, data.dtls
    # skiplist = ["TABLE:MULTIVARIABLELOOKUP"]
    nofirstfields = iddgaps.missingkeys_standard(commdct, dtls, 
                skiplist=["TABLE:MULTIVARIABLELOOKUP"]) 
    iddgaps.missingkeys_nonstandard(commdct, dtls, nofirstfields)
    bunchdt = makebunches(data, commdct)
    # TODO : add functions here.
    # - 
    addfunctions(dtls, bunchdt)
    # - 
    return bunchdt, data, commdct

def idfreader1(fname, iddfile, conv=True, commdct=None, block=None):
    """read idf file and reutrn bunches"""
    versiontuple = iddversiontuple(iddfile)
    block, data, commdct = readidf.readdatacommdct1(fname,
                                iddfile=iddfile, commdct=commdct, block=block)
    if conv:
        convertallfields(data, commdct)
    # fill gaps in idd
    dt, dtls = data.dt, data.dtls
    if versiontuple < (8, ):
        skiplist = ["TABLE:MULTIVARIABLELOOKUP"]
    else:
        skiplist = None
    nofirstfields = iddgaps.missingkeys_standard(commdct, dtls, 
                skiplist=skiplist) 
    iddgaps.missingkeys_nonstandard(commdct, dtls, nofirstfields)
    bunchdt = makebunches(data, commdct)
    # TODO : add functions here.
    # - 
    addfunctions(dtls, bunchdt)
    # - 
    return bunchdt, block, data, commdct

# class IIDF0(object):
#     iddname = None
#     def __init__(self, idfname):
#         self.idfname = idfname
#         self.read()
#     @classmethod
#     def setiddname(cls, arg):
#         if cls.iddname == None:
#             cls.iddname = arg
#             cls.idd_info = None
#             cls.block = None
#     @classmethod
#     def setidd(cls, iddinfo, block):
#         cls.idd_info = iddinfo
#         cls.block = block
#     def read(self):
#         """read the idf file"""
#         # TODO unit test
#         # TODO : thow an exception if iddname = None
#         readout = idfreader1(self.idfname, self.iddname,
#                                 commdct=self.idd_info, block=self.block)
#         self.idfobjects, block, self.model, idd_info = readout
#         self.__class__.setidd(idd_info, block)
#     def save(self):
#         # TODO unit test
#         s = str(self.model)
#         open(self.idfname, 'w').write(s)
#     def saveas(self, filename):
#         s = str(self.model)
#         open(filename, 'w').write(s)
# 
# class IIDF1(IIDF0):
#     def __init__(self, idfname):
#         super(IIDF1, self).__init__(idfname)
#     def newidfobject(self, key, aname=''):
#         """add a new idfobject to the model"""
#         # TODO unit test
#         return addobject(self.idfobjects,
#                             self.model,
#                             self.idd_info,
#                             key, aname=aname)  
#     def addidfobject(self, idfobject):
#         """add idfobject to this model"""
#         # TODO unit test
#         addthisbunch(self.model,
#                             self.idd_info,
#                             idfobject)  
# class IIDF2(IIDF1):
#     def __init__(self, idfname):
#         super(IIDF2, self).__init__(idfname)
#     def idfstr(self):
#         # print self.model.__repr__()
#         st = ''
#         dtls = self.model.dtls
#         for objname in dtls:
#             for obj in self.idfobjects[objname]:
#                  st = st + obj.__repr__()
#         return st
#     def printidf(self):
#         """print the idf"""
#         s = self.idfstr()
#         dtls = self.model.dtls
#         for objname in dtls:
#             for obj in self.idfobjects[objname]:
#                  print obj
#     # def __repr__(self):
#     #     return self.model.__repr__()
# 
# IIDF = IIDF2

# read code
# iddfile = "../iddfiles/Energy+V6_0.idd"
# fname = "../idffiles/5ZoneSupRetPlenRAB.idf" # small file with only surfaces
# bunchdt, data, commdct = idfreader(fname, iddfile)
# 
# zones = bunchdt['zone'.upper()]
# surfaces = bunchdt['BUILDINGSURFACE:DETAILED'.upper()]
# currentobjs = [key for key in bunchdt.keys() if len(bunchdt[key]) > 0]
# 
# print data


# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

# Find the intersection between two lines
# V = (1/6)*|(a-d).((b-d)x(c-d))|
import numpy as np
def vol_tehrahedron(poly):
    """volume of a irregular tetrahedron"""
    a = np.array(poly[0])
    b = np.array(poly[1])
    c = np.array(poly[2])
    d = np.array(poly[3])
    return abs(np.dot((a-d),np.cross((b-d),(c-d))) / 6)

poly = [(0.0,0.0,0.0),(1.0,0.0,0.0),(1.0,1.0,0.0),(0.0,0.0,1.0)]

def central_p(poly1,poly2):
    """central point of a prism"""
    central_point = np.array([0.0,0.0,0.0])
    for i in range(len(poly1)):
        central_point += np.array(poly1[i]) + np.array(poly2[i])
    return central_point/ (len(poly1)) / 2

poly1 = [(0.0,0.0,0.0),(1.0,0.0,0.0),(1.0,1.0,0.0),(0,1.0,0.0)]
poly2 = [(0.0,0.0,1),(1.0,0.0,1),(1.0,1.0,1),(0,1.0,1)]

def vol_zone(poly1,poly2):
    """"volume of a zone defined by two polygon bases """
    c_point = central_p(poly1,poly2)
    c_point = (c_point[0],c_point[1],c_point[2])
    vol_therah = 0
    N = len(poly1)
    for i in range(N-2):
        # the upper part
        tehrahedron = [c_point,poly1[0],poly1[i+1],poly1[i+2]]
        vol_therah += vol_tehrahedron(tehrahedron)
        # the bottom part
        tehrahedron = [c_point,poly2[0],poly2[i+1],poly2[i+2]]
        vol_therah += vol_tehrahedron(tehrahedron)
    # the middle part
    for i in range(N-1):
        tehrahedron = [c_point,poly1[i],poly2[i],poly2[i+1]]
        vol_therah += vol_tehrahedron(tehrahedron)
        tehrahedron = [c_point,poly1[i],poly1[i+1],poly2[i]]
        vol_therah += vol_tehrahedron(tehrahedron)
    tehrahedron = [c_point,poly1[N-1],poly2[N-1],poly2[0]]
    vol_therah += vol_tehrahedron(tehrahedron)
    tehrahedron = [c_point,poly1[N-1],poly1[0],poly2[0]]
    vol_therah += vol_tehrahedron(tehrahedron)
    return vol_therah
    



# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

"""some init routines to do the documentation"""

from IPython.display import Image
from IPython.display import display_pretty, display_html, display_jpeg, display_png, display_json, display_latex, display_svg
idfeditor = Image(filename='../docs/images/idf3.png')
material_lists = Image(filename='../docs/images/material_lists.png')
html_snippet1 = Image(filename='../docs/images/html1.png')
html_snippet2 = Image(filename='../docs/images/html2.png')
plantloop1 = Image(filename='../docs/images/hhh1.png')
plantloop2 = Image(filename='../docs/images/hhh_new.png')
diffcsv = Image(filename='../docs/images/diffcsv.tiff')


"""read the idf file by just parsing the text"""

import StringIO
import eppy.modeleditor as modeleditor
from eppy.modeleditor import IDF

def nocomment(st,com='!'):
    """
    just like the comment in python.
    removes any text after the phrase 'com'
    """
    ls=st.splitlines()
    for i in range(len(ls)):
        el=ls[i]
        pt=el.find(com)
        if pt!=-1:
            ls[i]=el[:pt]
    return '\n'.join(ls)


def _tofloat(s):
    try:
        return float(s)
    except ValueError as e:
        return s
        
def idf2txt(txt):
    """convert the idf text to a simple text"""
    st = nocomment(txt)
    objs = st.split(';')
    objs = [obj.split(',') for obj in objs]
    objs = [[line.strip() for line in obj] for obj in objs]
    objs = [[_tofloat(line) for line in obj] for obj in objs]
    objs = [tuple(obj) for obj in objs]
    objs.sort()

    lst = []
    for obj in objs:
        for field in obj[:-1]:
            lst.append('%s,' % (field, ))
        lst.append('%s;\n' % (obj[-1], ))

    return '\n'.join(lst)
    
    
def idfreadtest(iddhandle, idfhandle1, idfhandle2, verbose=False, save=False):
    """compare the results of eppy reader and simple reader"""
    # read using eppy:
    try:
        IDF.setiddname(iddhandle)
    except modeleditor.IDDAlreadySetError as e:
        # idd has already been set
        pass
    idf = IDF(idfhandle1)
    idfstr = idf.idfstr()
    idfstr = idf2txt(idfstr)
    # - 
    # do a simple read
    simpletxt = idfhandle2.read()
    simpletxt = idf2txt(simpletxt)
    # - 
    if save:
        open('simpleread.idf', 'w').write(idfstr)
        open('eppyread.idf', 'w').write(simpletxt)
    # do the compare      
    lines1 = idfstr.splitlines()
    lines2 = simpletxt.splitlines()
    for i, (line1, line2) in enumerate(zip(lines1, lines2)):
        if line1 != line2:
            # test if it is a mismatch in number format
            try:
                line1 = float(line1[:-1])
                line2 = float(line2[:-1])
                if line1 != line2:
                    if verbose:
                        print
                        print "%s- : %s" % (i, line1)
                        print "%s- : %s" % (i, line2)
                    return False
            except ValueError as e:
                if verbose:
                    print
                    print "%s- : %s" % (i, line1)
                    print "%s- : %s" % (i, line2)
                return False    
    return True
    



# fname = './iddfile/smallfile.idf'
# fhandle = open(fname, 'r')
# st = fhandle.read()
# print idf2txt(st)


# EPlusInterface (EPI) - An interface for EnergyPlus
# Copyright (C) 2004 Santosh Philip

# This file is part of EPlusInterface.
# 
# EPlusInterface is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# EPlusInterface is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with EPlusInterface; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 


# Santosh Philip, the author of EPlusInterface, can be contacted at the following email address:
# santosh_philip AT yahoo DOT com
# Please send all bug reports, enhancement proposals, questions and comments to that address.
# 
# VERSION: 0.001


import mylib1,mylib2
import string


def transpose2D(ls):
#transpose a 2D array. actually a list like
#[[a,b],[c,d],[e,f]]->[[a,c,e],[b,d,f]]
	outer=[]
	for i in range(len(ls[0])):
		inner=[]
		for j in range(len(ls)):
			inner=inner+[ls[j][i]]
		outer=outer+[inner]
	return outer



def getfilelist(flist,lsep):
#flist is s text file with a list of file names
#the file names are returned in a list
	st=mylib1.readfile(flist)
	st=st.rstrip()
	ls=st.split(lsep)
	return ls
	

dossep='\r\n'
macsep='\r'
unixsep='\n'

def getlinesep(st):
#returns the line seperators used in the string st
#mac is '\r'
#dos is '\r\n'
#unix is '\n'
	lsep=None
	if len(st)==0:lsep=None
	r=string.count(st,'\r')
	n=string.count(st,'\n')
	if n==r==0:lsep=None
	if n==0:lsep='\r'
	if r==0:lsep='\n'
	if n==r:lsep='\r\n'
	return lsep

#closefloat
def closefloat(f1,f2,closeness=0.01):
	#Check if the two floating points are close to each other
	dif=abs(f1-f2)
	return dif<closeness




# A simple HTML table parser. It turns tables (including nested tables) into arrays
# Nigel Sim <nigel.sim@gmail.com>
# http://simbot.wordpress.com
from HTMLParser import HTMLParser
import re, string, os
from string import lower

class Table(list):
    pass
	
class Row(list):
    pass

class Cell(object):
    def __init__(self):
        self.data = None
        return
    def append(self,item):
        if self.data != None:
	    print "Overwriting %s"%self.data
        self.data = item

# Get the item on the top of a stack
def top(x):
    return x[len(x)-1]

class TableParser(HTMLParser):
    def __init__(self, parser=None):
        """
	The parser is a method which will be passed the doc at the end
	of the parsing. Useful if TableParser is within an inner loop and
	you want to automatically process the document. If it is omitted then
	it will do nothing
	"""
        self._tag = None
	self._buf = None
	self._attrs = None
	self.doc = None # Where the document will be stored
	self._stack = None
	self._parser = parser
	self.reset()
        return

    def reset(self):
        HTMLParser.reset(self)
	self.doc = []
	self._stack = [self.doc]
	self._buf = ''

    def close(self):
        HTMLParser.close(self)
	if self._parser != None:
	    self._parser(self.doc)

    def handle_starttag(self, tag, attrs):
        self._tag = tag
	self._attrs = attrs
	if lower(tag) == 'table':
	    self._buf = ''
            self._stack.append(Table())
	elif lower(tag) == 'tr':
	    self._buf = ''
            self._stack.append(Row())
	elif lower(tag) == 'td':
	    self._buf = ''
            self._stack.append(Cell())
	
        #print "Encountered the beginning of a %s tag" % tag

    def handle_endtag(self, tag):
	if lower(tag) == 'table':
	    t = None
	    while not isinstance(t, Table):
                t = self._stack.pop()
	    r = top(self._stack)
            r.append(t)

	elif lower(tag) == 'tr':
	    t = None
	    while not isinstance(t, Row):
                t = self._stack.pop()
	    r = top(self._stack)
            r.append(t)

	elif lower(tag) == 'td':
	    c = None
	    while not isinstance(c, Cell):
                c = self._stack.pop()
	    t = top(self._stack)
	    if isinstance(t, Row):
	        # We can not currently have text and other table elements in the same cell. 
		# Table elements get precedence
	        if c.data == None:
                    t.append(self._buf)
		else:
		    t.append(c.data)
	    else:
	        print "Cell not in a row, rather in a %s"%t
        self._tag = None
        #print "Encountered the end of a %s tag" % tag

    def handle_data(self, data):
        self._buf += data




#!/usr/bin/env python

## EPlusInterface (EPI) - An interface for EnergyPlus
## Copyright (C) 2004 Santosh Philip
##
## This file is part of EPlusInterface.
## 
## EPlusInterface is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## EPlusInterface is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with EPlusInterface; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
##
##
## Santosh Philip, the author of EPlusInterface, can be contacted at the following email address:
## santosh_philip AT yahoo DOT com
## Please send all bug reports, enhancement proposals, questions and comments to that address.
## 
## VERSION: 0.001

import sys
import getopt
import os
import string
import user

sys.path.append(user.home+'/Library/PythonFiles')
import mylib1,mylib2,mylib3
import eplus2,time
import parse_idd

# block,commlst,commdct=parse_idd.extractidddata('./Energy+.idd')
# 
# cm=commdct[3]
# cmm=cm[0]

def printcdt(i):
	for el in comm[i]:
		print el



# commtype=['field', 'note', 'required-field', 'units', 'ip-units', 'minimum', 
# 'minimum>', 'maximum', 'maximum<', 'default', 'autosizable', 'type', 
# 'key', 'object-list', 'reference', 
# 'memo', 'unique-object', 'required-object', 'min-fields', 
# 'obsolete', 'extensible', 
# 'group']


		

class slashcomment:
	def __init__(self):
		self.allkeys=['field', 'note', 'required-field', 'units', 'ip-units', 'minimum', 
			'minimum>', 'maximum', 'maximum<', 'default', 'autosizable', 'type', 
			'key', 'object-list', 'reference', 
			'memo', 'unique-object', 'required-object', 'min-fields', 
			'obsolete', 'extensible', 
			'group']
		self.commentnow={
			'default': ['4'], 
			'maximum': ['6'], 
			'required-field': [''], 
			'note': [
				'Number in hour: validity 1 to 6: 4 suggested', 
				'Should be evenly divisible into 60', 
				'Specifying 6 as maximum as higher values may cause instability.'], 
			'field': ['Time Step in Hour'], 
			'minimum': ['1'], 
			'type': ['integer']}
	
	def keyshere(self,cmm):
		ls=[]
		for el in self.allkeys:
			if cmm.has_key(el):
				ls.append(el)
		
		return ls
	


def checkNumber(num):
	"""
	check if num is a number 
	and raise an error
	"""
	try:
		return float(num)
	except:
		raise "anError", "%s is not a real"%(num) 

def checkInteger(num):
	"""
	check if num is an integer 
	and raise an error
	"""
	try:
		return int(num)
	except:
		raise "anError", "%s is not an integer"%(num) 
		

def getobjectindex(blocklst,object):
	"""
	getobjectindex(blocklst,'SOLUTION ALGORITHM')
	returns the index of the object
	corresponding slashcomments can be retrived using this index
	"""
	ls=[]
	for el in blocklst:
		ls.append(el[0].upper())
	return ls.index(object.upper())



def getrepeats():
	"""
	no docs - 
	I used it for debugging
	to find how many items had 'field' in more than one line
	"""
	key='reference'
	for i in range(len(commdct)):
		for j in range(len(commdct[i])):
			try:
				val=commdct[i][j][key]
				if len(val)>1:
					print commdct[i][j]
					print i,j,'====',len(val)
			except:
				pass
		
	
def checkVal(val,commentdct=None):
	"""
	raise an error if the value does not fall into 
	the description of commentdct
	"""
	# check autosizable
	# check type - real, integer
	# check min max min> max<
	# check key
	# check object-list
	
	
	error="anError"
	#check on autosize - return if value is autosize
	if commentdct.has_key('autosizable'):
		print 'autosizeable'
		if val=='autosize':
			return
	
	#check type - initially check number
	keyval=getkeyval('type',commentdct)
	if keyval!=None:
		if keyval.upper()=='real'.upper():
			val=checkNumber(val)
	if keyval!=None:
		if keyval.upper()=='integer'.upper():
			val=checkInteger(val)
	
	# check min max min> max<
	keyval=getkeyval('minimum',commentdct)
	if keyval!=None:
		if not val>=keyval:
			raise error, "value should be greater than or equal to %s"%(keyval)
	keyval=getkeyval('minimum>',commentdct)
	if keyval!=None:
		if not val>keyval:
			raise error, "value should be greater than %s"%(keyval)
	keyval=getkeyval('maximum',commentdct)
	if keyval!=None:
		if not val<=keyval:
			raise error, "value should be less than or equal to %s"%(keyval)
	keyval=getkeyval('maximum<',commentdct)
	if keyval!=None:
		if not val<keyval:
			raise error, "value should be less than%s"%(keyval)
	
	# check key
	keyval=getkeyval('key',commentdct)
	if keyval!=None:
		try:
			keyval.index(val)
		except:
			raise "anError", "%s is not in list" %(val,)
		



def getkeyval(key,commentdct):
	"""
	returns the value of the key
	commentdct can be {'default': ['4'], 'maximum': ['6'], 'required-field': [''],
				'note':['this is', ' a note']}
	now key='default' will return the number 4
	key='note' will return 'this is\\n a note'
	"""
	#--------minimum, minimum>, maximum, maximum<, min-fields----------
	#returns the number or None
	if key.upper() in [
				'minimum'.upper(),
				'minimum>'.upper(),
				'maximum'.upper(),
				'maximum<'.upper(),
				'min-fields'.upper()]:
		try:
			val=commentdct[key][0]
			return float(val)
		except KeyError:
			return None
	#--------field, note, units, ip-units, 
	#        default, type, object-list, memo----------
	#returns the value or None
	if key.upper() in [
				'field'.upper(),
				'note'.upper(),
				'units'.upper(),
				'ip-units'.upper(),
				'default'.upper(),
				'type'.upper(),
				'object-list'.upper(),
				'memo'.upper()]:
		try:
			val=commentdct[key]
			st=''
			for el in val:
				st=st+el+'\n'
			st=st[:-1]
		except KeyError:
			return None
		return st
	#--------required-field, autosizable, unique-object, required-object----------
	#returns True or False
	if key.upper()in [
				'required-field'.upper(), 
				'autosizable'.upper(),
				'unique-object'.upper(),
				'required-object'.upper()]:
		return commentdct.has_key(key)
	#--------key, reference----------
	#returns a list
	if key.upper()in [
				'key'.upper(),
				'reference'.upper()]:
		try:
			return commentdct[key]
		except KeyError:
			return None




#key var -> to run getkeyval
# i=7
# j=0
# print block[i], block[i][j]
# cm=commdct[i]
# key='memo'
# print getkeyval(key,cm[j])

# block,commlst,commdct=parse_idd.extractidddata('./Energy+.idd')
# commentdct={
# 	'units': ['deltaC'], 
# 	'field': ['Temperature Convergence Tolerance Value'], 
# 	'minimum>': ['0.0'], 
# 	'type': ['real'], 'default': ['.4']}
# 
# 
# 
# checkVal(-5,commentdct)
# 
# 
# try:
# 	checkVal(-2,commentdct)	
# except "anError",d:
# 	print d
# 
# 
# try:
# 	checkNumber('a')	
# except "anError",d:
# 	print d
# 	



from __future__ import print_function
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import io
import codecs
import os
import sys

import eppy

here = os.path.abspath(os.path.dirname(__file__))

def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)

long_description = read('README.txt')

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errcode = pytest.main(self.test_args)
        sys.exit(errcode)

setup(
    name='eppy',
    version=eppy.__version__,
    url='https://github.com/santoshphilip/eppy',
    license='GNU AFFERO GENERAL PUBLIC LICENSE',
    author='Santosh Philip',
    tests_require=['pytest'],
    install_requires=["bunch>=1.0",
                "beautifulsoup4>=4.2.1",
                # "numpy>=1.7.1",
                "pyparsing>=pyparsing",
                "pydot>1.0",
                "pytest>=2.3.5",
                    ],
    cmdclass={'test': PyTest},
    author_email='eppy_scripting@yahoo.com',
    description='Scripting language for E+ idf files, and E+ output files',
    long_description=long_description,# TODO set this up
    packages=['eppy', 'eppy.EPlusInterfaceFunctions', 'eppy.geometry'],
    include_package_data=True,
    platforms='any',
    test_suite='eppy.test.test_eppy',# TODO make test_eppy
    classifiers = [
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering',
        ],
    extras_require={
        'testing': ['pytest'],
    }
)


# EPlusInterface (EPI) - An interface for EnergyPlus
# Copyright (C) 2004 Santosh Philip

# This file is part of EPlusInterface.
# 
# EPlusInterface is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# EPlusInterface is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with EPlusInterface; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 


# Santosh Philip, the author of EPlusInterface, can be contacted at the following email address:
# santosh_philip AT yahoo DOT com
# Please send all bug reports, enhancement proposals, questions and comments to that address.
# 
# VERSION: 0.001

import os,pickle,cPickle
import string
import mylib1


ret='\r\n'

def readfile(filename):
    f=open(filename,'rb')
    data=f.read()
    f.close()
    return data

def printlist(ls):
    for n in range(0,len(ls)):
        print ls[n]

def printdict(d):
    dl=d.keys()
    dl.sort()
    for i in range(0,len(dl)):
        print dl[i],d[dl[i]]

def tabfile2list(fname):
    #dat=mylib1.readfileasmac(fname)
    #data=string.strip(dat)
    data=mylib1.readfileasmac(fname)
    #data=data[:-2]#remove the last return
    alist=string.split(data,'\r')#since I read it as a mac file
    blist=string.split(alist[1],'\t')

    clist=[]
    for n in range(0,len(alist)):
        ilist=string.split(alist[n],'\t')
        clist=clist+[ilist]
    cclist=clist[:-1]#the last element is turning out to be empty
    return cclist

def tabstr2list(data):
    alist=string.split(data,os.linesep)
    blist=string.split(alist[1],'\t')

    clist=[]
    for n in range(0,len(alist)):
        ilist=string.split(alist[n],'\t')
        clist=clist+[ilist]
    cclist=clist[:-1]
      #the last element is turning out to be empty
      #this is because the string ends with a os.linesep
    return cclist

def list2doe(l):
    eq=''
    str=''
    lenj=len(l)
    leni=len(l[0])
    for i in range(0,leni-1):
        for j in range(0,lenj):
            if j==0:
                str=str+l[j][i+1]+eq+l[j][0]+ret
            else:
                str=str+l[j][0]+eq+l[j][i+1]+ret
        str=str+ret
    return str

def tabfile2doefile(tabfile,doefile):
    alist=tabfile2list(tabfile)
    str=list2doe(alist)
    mylib1.writeStr2File(doefile,str)

def tabstr2doestr(st):
    alist=tabstr2list(st)
    str=list2doe(alist)
    return str

def makedoedict(str1):
    blocklist=string.split(str1,'..')
    blocklist=blocklist[:-1]#remove empty item after last '..'
    blockdict={}
    belongsdict={}
    for n in range(0,len(blocklist)):
        blocklist[n]=blocklist[n].strip()
        linelist=string.split(blocklist[n],os.linesep)
        aline=linelist[0]
        alinelist=string.split(aline,'=')
        name=alinelist[0].strip()
        aline=linelist[1]
        alinelist=string.split(aline,'=')
        belongs=alinelist[-1].strip()
        theblock=blocklist[n]+os.linesep+'..'+os.linesep+os.linesep
            #put the '..' back in the block
        blockdict[name]=theblock
        belongsdict[name]=belongs
    return [blockdict,belongsdict]

def makedoetree(d,b):
    dl=d.keys()
    bl=b.keys()
    dl.sort()
    bl.sort()
    
    #make space dict
    doesnot='DOES NOT'
    lst=[]
    for n in range(0,len(bl)):
        if b[bl[n]]==doesnot:#belong
            lst=lst+[bl[n]]
    
    doedict={}    
    for n in range(0,len(lst))   :
        #print lst[n]
        doedict[lst[n]]={}
    lv1list=doedict.keys()
    lv1list.sort()
    
    #make wall dict
    #for each space
    for i in range(0,len(lv1list)):
        walllist=[]
        dict=doedict[lv1list[i]]
        #loop thru the entire bl dictonary and list the ones that belong into walllist
        for n in range(0,len(bl)):
            if b[bl[n]]==lv1list[i]:
                walllist=walllist+[bl[n]]
        #put walllist into dict
        for j in range(0,len(walllist)):
            dict[walllist[j]]={}
    
    #make window dict
    #for each space
    for i in range(0,len(lv1list)):
        dict1=doedict[lv1list[i]]
        #for each wall
        walllist=dict1.keys()
        walllist.sort()
        for j in range(0,len(walllist)):
            windlist=[]
            dict2=dict1[walllist[j]]
           #loop thru the entire bl dictonary and list the ones that belong into windlist
            for n in range(0,len(bl)):
                if b[bl[n]]==walllist[j]:
                    windlist=windlist+[bl[n]]
            #put walllist into dict
            for k in range(0,len(windlist)):
                dict2[windlist[k]]={}
    return doedict
                
def tree2doe(str1):
    retstuff=makedoedict(str1)
    ddict= makedoetree(retstuff[0],retstuff[1])
    d=retstuff[0]
    retstuff[1] ={}# don't need it anymore

    str1=''#just re-using it
    L1list=ddict.keys()
    L1list.sort()
    for i in range(0,len(L1list)):
        str1=str1+d[L1list[i]]
        L2list=ddict[L1list[i]].keys()
        L2list.sort()
        for j in range(0,len(L2list)):
            str1=str1+d[L2list[j]]
            L3list=ddict[L1list[i]][L2list[j]].keys()
            L3list.sort()
            for k in range(0,len(L3list)):
                str1=str1+d[L3list[k]]
    return str1

def Mtabstr2doestr(st1):
    seperator='$=================='
    ls=string.split(st1,seperator)

    #this removes all the tabs that excel
    #puts after the seperator and before the next line
    for n in range(0,len(ls)):
        ls[n]=string.lstrip(ls[n])
    st2=''
    for n in range(0,len(ls)):
        alist=tabstr2list(ls[n])
        st2=st2+list2doe(alist)

    lss=string.split(st2,'..')
    mylib1.writeStr2File('forfinal.txt',st2)#for debugging
    print len(lss)


    st3=tree2doe(st2)
    lsss=string.split(st3,'..')
    print len(lsss)
    return st3

def getoneblock(st,start,end):
#get the block bounded by start and end
#doesn't work for multiple blocks
    ls=string.split(st,start)
    st=ls[-1]
    ls=string.split(st,end)
    st=ls[0]
    return st

def doestr2tabstr(st,kword):
    ls=string.split(st,'..')
    del st
    #strip junk put .. back
    for n in range(0,len(ls)):
        ls[n]=string.strip(ls[n])
        ls[n]=ls[n]+os.linesep+'..'+os.linesep
    ls.pop()

    lblock=[]
    for n in range(0,len(ls)):
        linels=string.split(ls[n],os.linesep)
        firstline=linels[0]
        assignls=string.split(firstline,'=')
        keyword=string.strip(assignls[-1])
        if keyword==kword:
            lblock=lblock+[ls[n]]
            #print firstline

    #get all val
    lval=[]
    for n in range(0,len(lblock)):
        block=lblock[n]
        linel=string.split(block,os.linesep)
        lvalin=[]
        for k in range(0,len(linel)):
            line=linel[k]
            assignl=string.split(line,'=')
            if k==0:
                lvalin=lvalin+[assignl[0]]
            else:
                if assignl[-1]=='..':
                    assignl[-1]='.'
                lvalin=lvalin+[assignl[-1]]
        lvalin.pop()
        lval=lval+[lvalin]

    #get keywords
    kwordl=[]
    block=lblock[0]
    linel=string.split(block,os.linesep)
    for k in range(0,len(linel)):
        line=linel[k]
        assignl=string.split(line,'=')
        if k==0:
            kword='= '+string.strip(assignl[1])
        else:
            if assignl[0]=='..':
                assignl[0]='.'
            else:
                assignl[0]=assignl[0]+'='
            kword=string.strip(assignl[0])       
        kwordl=kwordl+[kword]
    kwordl.pop()
    
    st=''
    for n in range(0,len(kwordl)):
        linest=''
        linest=linest+kwordl[n]
        for k in range(0,len(lval)):
            linest=linest+'\t'+lval[k][n]
        st=st+linest+os.linesep

    return st

def myreplace(s,f,r):
    "in string s replace all occurences of f with r"
    alist=string.split(s,f)
    new_s=string.join(alist,r)
    return new_s

def fslicebefore(s,sub):
    "Return the slice starting at sub in string s"
    f=string.find(s,sub)
    return s[f:]

def fsliceafter(s,sub):
    "Return the slice after at sub in string s"
    f=string.find(s,sub)
    return s[f+len(sub):]

def pickleload(fname):
    "same as pickle.load(f).takes filename as parameter"
    f=open(fname,'rb')
    return pickle.load(f)

def pickledump(o,fname):
    "same as pickle.dump(o,f).takes filename as parameter"
    f=open(fname,'wb')
    pickle.dump(o,f)

def cpickleload(fname):
    "same as pickle.load(f).takes filename as parameter"
    f=open(fname,'rb')
    return cPickle.load(f)

def cpickledump(o,fname):
    "same as pickle.dump(o,f).takes filename as parameter"
    f=open(fname,'wb')
    cPickle.dump(o,f)



# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

"""functions to edit the E+ model"""

from idfreader import makebunches
from idfreader import idfreader, idfreader1
from idfreader import makeabunch
import function_helpers
import copy

class NoObjectError(Exception):
    pass

class NotSameObjectError(Exception):
    pass

class IDDNotSetError(Exception):
    pass

class IDDAlreadySetError(Exception):
    pass

def almostequal(first, second, places=7, printit=True):
    # taken from python's unit test
    # may be covered by Python's license 
    """docstring for almostequal"""
    if round(abs(second-first), places) != 0:
        if printit:
            print round(abs(second-first), places)
            print "notalmost: %s != %s" % (first, second)
        return False
    else:
        return True
        
def poptrailing(lst):
    """pop the trailing items in lst that are blank"""
    for i in range(len(lst)):
        if lst[-1] != '':
            break
        lst.pop(-1)
    return lst
    
def extendlist(lst, i, value=''):
    """extend the list so that you have i-th value"""
    if i < len(lst):
        pass
    else:
        lst.extend([value, ] * (i - len(lst) + 1))

    

def newrawobject(data, commdct, key):
    """make a new object for key"""
    dt = data.dt
    dtls = data.dtls
    key = key.upper()

    key_i = dtls.index(key)
    key_comm = commdct[key_i]
    #set default values
    obj = [comm.get('default', [''])[0] for comm in key_comm] 
    for i, comm in enumerate(key_comm):
        typ = comm.get('type', [''])[0]
        if typ in ('real', 'integer'):
            func_select = dict(real=float, integer=int)
            try:
                obj[i] = func_select[typ](obj[i])
            except IndexError, e:
                break
            except ValueError, e: # if value = autocalculate
                continue
    obj[0] = key
    obj = poptrailing(obj) # remove the blank items in a repeating field. 
    return obj
    
def addthisbunch(bunchdt, data, commdct, thisbunch):
    """add a bunch to model.
    abunch usually comes from another idf file
    or it can be used to copy within the idf file"""
    key = thisbunch.key.upper()
    obj = copy.copy(thisbunch.obj)
    data.dt[key].append(obj)
    abunch =  obj2bunch(data, commdct, obj)
    bunchdt[key].append(abunch)
    return abunch
    
def obj2bunch(data, commdct, obj):
    """make a new bunch object using the data object"""
    dt = data.dt
    dtls = data.dtls
    key = obj[0].upper()
    key_i = dtls.index(key)
    abunch = makeabunch(commdct, obj, key_i)
    return abunch
    
def namebunch(abunch, aname):
    """give the bunch object a name, if it has a Name field"""
    if abunch.Name == None:
        pass
    else:
        abunch.Name = aname
    return abunch
    
def renamebunch(bunchdt, commdct, oldname, newname):
    """rename this bunch and change name in all references"""
    pass
    
def addobject(bunchdt, data, commdct, key, aname=None, **kwargs):
    """add an object to the eplus model"""
    obj = newrawobject(data, commdct, key)
    abunch = obj2bunch(data, commdct, obj)
    if aname:
        namebunch(abunch, aname)
    data.dt[key].append(obj)
    bunchdt[key].append(abunch)
    for k, v in kwargs.items():
        abunch[k] = v
    return abunch

def getnamedargs(*args, **kwargs):
    """allows you to pass a dict and named args
    so you can pass ({'a':5, 'b':3}, c=8) and get
    dict(a=5, b=3, c=8)"""
    adict = {}
    for arg in args:
        if isinstance(arg, dict):
            adict.update(arg)
    adict.update(kwargs)
    return adict
    
def addobject1(bunchdt, data, commdct, key, **kwargs):
    """add an object to the eplus model"""
    obj = newrawobject(data, commdct, key)
    abunch = obj2bunch(data, commdct, obj)
    data.dt[key].append(obj)
    bunchdt[key].append(abunch)
    # adict = getnamedargs(*args, **kwargs)
    for kkey, value in kwargs.iteritems():
        abunch[kkey] = value
    return abunch
    
def getobject(bunchdt, key, name):
    """get the object if you have the key and the name
    retunrs a list of objects, in case you have more than one
    You should not have more than one"""
    # TODO : throw exception if more than one object, or return more objects
    idfobjects = bunchdt[key]
    theobjs = [idfobj for idfobj in idfobjects if idfobj.Name.upper() == name.upper()]
    try:
        return theobjs[0]
    except IndexError, e:
        return None

def __objecthasfields(bunchdt, data, commdct, idfobject, places=7, **kwargs):
    """test if the idf object has the field values in kwargs"""
    key = idfobject.obj[0].upper()
    for k, v in kwargs.items():
        if not isfieldvalue(bunchdt, data, commdct, 
                idfobject, k, v, places=places):
            return False
    return True
        
def getobjects(bunchdt, data, commdct, key, places=7, **kwargs):
    """get all the objects of key that matches the fields in **kwargs"""
    idfobjects = bunchdt[key]
    allobjs = []
    for obj in idfobjects:
        if __objecthasfields(bunchdt, data, commdct, 
                obj, places=places, **kwargs):
            allobjs.append(obj)
    return allobjs
    
def iddofobject(data, commdct, key):
    """from commdct, return the idd of the object key"""
    dtls = data.dtls
    i = dtls.index(key)
    return commdct[i]
    
def getextensibleindex(bunchdt, data, commdct, key, objname):
    """get the index of the first extensible item"""
    theobject = getobject(bunchdt, key, objname)
    if theobject == None:
        return None
    theidd = iddofobject(data, commdct, key)
    extensible_i = [i for i in range(len(theidd)) if theidd[i].has_key('begin-extensible')]
    try:
        extensible_i = extensible_i[0]
    except IndexError, e:
        return theobject
        

def removeextensibles(bunchdt, data, commdct, key, objname):
    """remove the extensible items in the object"""
    theobject = getobject(bunchdt, key, objname)
    if theobject == None:
        return theobject
    theidd = iddofobject(data, commdct, key)
    extensible_i = [i for i in range(len(theidd)) if theidd[i].has_key('begin-extensible')]
    try:
        extensible_i = extensible_i[0]
    except IndexError, e:
        return theobject
    while True:
        try:
            p = theobject.obj.pop(extensible_i)
        except IndexError, e:
            break
    return theobject

def getfieldcomm(bunchdt, data, commdct, idfobject, fieldname):
    """get the idd comment for the field"""
    key = idfobject.obj[0].upper()
    keyi = data.dtls.index(key)
    fieldi = idfobject.objls.index(fieldname)
    thiscommdct = commdct[keyi][fieldi]
    return thiscommdct
    
def is_retaincase(bunchdt, data, commdct, idfobject, fieldname):
    """test if case has to be retained for that field"""
    thiscommdct = getfieldcomm(bunchdt, data, commdct, idfobject, fieldname)
    return thiscommdct.has_key('retaincase')
    
def isfieldvalue(bunchdt, data, commdct, idfobj, fieldname, value, places=7):
    """test if idfobj.field == value"""
    # do a quick type check
    # if type(idfobj[fieldname]) != type(value):
    #     return False # takes care of autocalculate and real
    # check float
    thiscommdct = getfieldcomm(bunchdt, data, commdct, idfobj, fieldname)
    if thiscommdct.has_key('type'):
        if thiscommdct['type'][0] in ('real', 'integer'):
            # test for autocalculate
            try:
                if idfobj[fieldname].upper() == 'AUTOCALCULATE':
                    if value.upper() == 'AUTOCALCULATE':
                        return True
            except AttributeError, e:
                pass
            return almostequal(float(idfobj[fieldname]), float(value), places, False)    
    # check retaincase
    if is_retaincase(bunchdt, data, commdct, idfobj, fieldname):
        return idfobj[fieldname] == value
    else:
        return idfobj[fieldname].upper() == value.upper()
    

def equalfield(bunchdt, data, commdct, idfobj1, idfobj2, fieldname, places=7):
    """returns true if the two fields are equal
    will test for retaincase
    places is used if the field is float/real"""
    # TODO test if both objects are of same type
    key1 = idfobj1.obj[0].upper()
    key2 = idfobj2.obj[0].upper()
    if key1 != key2:
        raise NotSameObjectError
    v2 = idfobj2[fieldname]
    return isfieldvalue(bunchdt, data, commdct, 
        idfobj1, fieldname, v2, places=places)
    
def getrefnames(idf, objname):
    """get the reference names for this object"""
    iddinfo = idf.idd_info
    dtls = idf.model.dtls
    index = dtls.index(objname)
    fieldidds = iddinfo[index]
    for fieldidd in fieldidds:
        if fieldidd.has_key('field'):
            if fieldidd['field'][0].endswith('Name'):
                if fieldidd.has_key('reference'):
                    return fieldidd['reference']
                else:
                    return []

def getallobjlists(idf, refname):
    """get all object-list fields for refname
    return a list:
    [('OBJKEY', refname, fieldindexlist), ...] where
    fieldindexlist = index of the field where the object-list = refname
    """
    dtls = idf.model.dtls
    objlists = []
    for i, fieldidds in enumerate(idf.idd_info):
        indexlist = []
        for j, fieldidd in enumerate(fieldidds):
            if fieldidd.has_key('object-list'):
                if fieldidd['object-list'][0].upper() == refname.upper():
                    indexlist.append(j)
        if indexlist != []:
            objkey = dtls[i]
            objlists.append((objkey, refname, indexlist))
    return objlists

def rename(idf, objkey, objname, newname):
    """rename all the refrences to this objname"""
    refnames = getrefnames(idf, objkey)
    for refname in refnames:
        objlists = getallobjlists(idf, refname) 
        # [('OBJKEY', refname, fieldindexlist), ...]
        for refname in refnames:
            for robjkey, refname, fieldindexlist in objlists:
                idfobjects = idf.idfobjects[robjkey]
                for idfobject in idfobjects:
                    for findex in fieldindexlist: # for each field
                        if idfobject[idfobject.objls[findex]] == objname:
                            idfobject[idfobject.objls[findex]] = newname
    theobject = idf.getobject(objkey, objname)
    fieldname = [item for item in theobject.objls if item.endswith('Name')][0]
    theobject[fieldname] = newname
    return theobject

def zonearea(idf, zonename, debug=False):
    zone = idf.getobject('ZONE', zonename)
    surfs = idf.idfobjects['BuildingSurface:Detailed'.upper()]
    zone_surfs = [s for s in surfs if s.Zone_Name == zone.Name]
    floors = [s for s in zone_surfs if s.Surface_Type.upper() == 'FLOOR']
    if debug:
        print len(floors)
        print [floor.area for floor in floors]
    # area = sum([floor.area for floor in floors])
    if floors != []:
        area = zonearea_floor(idf, zonename)
    else:
        area = zonearea_roofceiling(idf, zonename)
    return area
    
def zonearea_floor(idf, zonename, debug=False):
    zone = idf.getobject('ZONE', zonename)
    surfs = idf.idfobjects['BuildingSurface:Detailed'.upper()]
    zone_surfs = [s for s in surfs if s.Zone_Name == zone.Name]
    floors = [s for s in zone_surfs if s.Surface_Type.upper() == 'FLOOR']
    if debug:
        print len(floors)
        print [floor.area for floor in floors]
    area = sum([floor.area for floor in floors])
    return area

def zonearea_roofceiling(idf, zonename, debug=False):
    zone = idf.getobject('ZONE', zonename)
    surfs = idf.idfobjects['BuildingSurface:Detailed'.upper()]
    zone_surfs = [s for s in surfs if s.Zone_Name == zone.Name]
    floors = [s for s in zone_surfs if s.Surface_Type.upper() in ['ROOF', 
                                                            'CEILING']]
    if debug:
        print len(floors)
        print [floor.area for floor in floors]
    area = sum([floor.area for floor in floors])
    return area
    
def zone_height_min2max(idf, zonename, debug=False):
    zone = idf.getobject('ZONE', zonename)
    surfs = idf.idfobjects['BuildingSurface:Detailed'.upper()]
    zone_surfs = [s for s in surfs if s.Zone_Name == zone.Name]
    surf_xyzs = [function_helpers.getcoords(s) for s in zone_surfs]
    import itertools # to flatten the list
    surf_xyzs = list(itertools.chain(*surf_xyzs))
    surf_zs = [z for x, y, z in surf_xyzs]
    topz =  max(surf_zs)
    botz = min(surf_zs)
    height = topz - botz    
    return height
    
def zoneheight(idf, zonename, debug=False):
    zone = idf.getobject('ZONE', zonename)
    surfs = idf.idfobjects['BuildingSurface:Detailed'.upper()]
    zone_surfs = [s for s in surfs if s.Zone_Name == zone.Name]
    floors = [s for s in zone_surfs if s.Surface_Type.upper() == 'FLOOR']
    roofs = [s for s in zone_surfs if s.Surface_Type.upper() == 'ROOF']
    if floors == [] or roofs == []:
        height = zone_height_min2max(idf, zonename)    
    else:
        height = zone_floor2roofheight(idf, zonename)    
    return height
    
def zone_floor2roofheight(idf, zonename, debug=False):
    zone = idf.getobject('ZONE', zonename)
    surfs = idf.idfobjects['BuildingSurface:Detailed'.upper()]
    zone_surfs = [s for s in surfs if s.Zone_Name == zone.Name]
    floors = [s for s in zone_surfs if s.Surface_Type.upper() == 'FLOOR']
    roofs = [s for s in zone_surfs if s.Surface_Type.upper() == 'ROOF']
    ceilings = [s for s in zone_surfs if s.Surface_Type.upper() == 'CEILING']
    topsurfaces = roofs + ceilings

    topz = []
    for topsurface in topsurfaces:
        for coord in topsurface.coords:
            topz.append(coord[-1])
    topz = max(topz)
        
    botz = []
    for floor in floors:
        for coord in floor.coords:
            botz.append(coord[-1])
    botz = min(botz)
    height = topz - botz
    
    return height
    
    
    
def zonevolume(idf, zonename):
    area = zonearea(idf, zonename)
    height = zoneheight(idf, zonename)
    volume = area * height

    return volume


class IDF0(object):
    """
    document the following variables:
    
    - idfobjects
    - outputtype
    - iddname
    - idfname
    - idd_info
    - model
    
    
"""
    iddname = None
    idd_info = None
    block = None
    def __init__(self, idfname=None):
        if idfname != None:
            self.idfname = idfname
            self.read()
    @classmethod
    def setiddname(cls, arg, testing=False):
        if cls.iddname == None:
            cls.iddname = arg
            cls.idd_info = None
            cls.block = None
        else:
            if testing == False:
                errortxt = "IDD file is set to: %s"  % (cls.iddname, )
                raise IDDAlreadySetError(errortxt)
    @classmethod
    def getiddname(cls):
        return cls.iddname
    @classmethod
    def setidd(cls, iddinfo, block):
        cls.idd_info = iddinfo
        cls.block = block
    def read(self):
        """read the idf file"""
        # TODO unit test
        if self.getiddname() == None:
            errortxt = "IDD file needed to read the idf file. Set it using IDF.setiddname(iddfile)"
            raise IDDNotSetError(errortxt)
        readout = idfreader1(self.idfname, self.iddname,
                                commdct=self.idd_info, block=self.block)
        self.idfobjects, block, self.model, idd_info = readout
        self.__class__.setidd(idd_info, block)
    def save(self):
        # TODO unit test
        s = str(self.model)
        open(self.idfname, 'w').write(s)
    def saveas(self, filename):
        s = str(self.model)
        open(filename, 'w').write(s)

class IDF1(IDF0):
    """subclass of IDF0. Uses functions of IDF0 
    """
    def __init__(self, idfname=None):
        super(IDF1, self).__init__(idfname)
    def newidfobject(self, key, aname='', **kwargs):
    # def newidfobject(self, key, *args, **kwargs):
        """add a new idfobject to the model
        
        for example :: 
        
            newidfobject("CONSTRUCTION")
            newidfobject("CONSTRUCTION", 
                Name='Interior Ceiling_class', 
                Outside_Layer='LW Concrete', 
                Layer_2='soundmat')
        
        If you don't specify a value for a field, the default value will be set
        
        aname is not used. It is left there for backward compatibility"""
        # TODO unit test
        # return addobject1(self.idfobjects,
        #                     self.model,
        #                     self.idd_info,
        #                     key, **kwargs)  
        return addobject(self.idfobjects,
                            self.model,
                            self.idd_info,
                            key, aname=aname, **kwargs)
    def popidfobject(self, key, index):
        """pop this object"""
        popobject = self.idfobjects[key][index]
        self.removeidfobject(popobject)
    def removeidfobject(self, idfobject):
        """remove this idfobject"""
        # the object has to be removed from idfobjects and
        # form self.model.dt
        # since idfobjects is just a wrapper for model.dt
        key = idfobject.key
        theobjects = self.idfobjects[key.upper()]
        for i, theobject in enumerate(theobjects):
            if theobject is idfobject:
                theobjects.pop(i)
                # remove it from model too
                return self.model.dt[key.upper()].pop(i)
    def copyidfobject(self, idfobject):
        """add idfobject to this model
        
        idfobject usually comes from another idf file
        or it can be used to copy within this idf file"""
        # TODO unit test
        addthisbunch(self.idfobjects, 
                            self.model,
                            self.idd_info,
                            idfobject)
    def getobject(self, key, name):
        """return the object given key and name"""
        return getobject(self.idfobjects, key, name)
    def getextensibleindex(self, key, name):
        """get the index of the first extensible item
        
        only for internal use. # TODO : hide this"""
        return getextensibleindex(self.idfobjects, self.model, self.idd_info, 
                                key, name)
    def removeextensibles(self, key, name):
        """remove extensible items in the object of key and name
        
        only for internal use. # TODO : hide this"""
        return removeextensibles(self.idfobjects, self.model, self.idd_info, 
                                key, name)
      
class IDF2(IDF1):
    """subclass of IDF1. Uses functions of IDF1 
    
    """
    def __init__(self, idfname=None):
        super(IDF2, self).__init__(idfname)
        self.outputtype = "standard" # standard, 
                                    # nocomment, 
                                    # nocomment1,
                                    # nocomment2,
                                    # compressed
    def idfstr(self):
        if self.outputtype != 'standard':
            st = self.model.__repr__()
            if self.outputtype == 'nocomment':
                return st
            elif self.outputtype == 'nocomment1':
                slist = st.split('\n')
                slist = [item.strip() for item in slist]
                return '\n'.join(slist)
            elif self.outputtype == 'nocomment2':
                slist = st.split('\n')
                slist = [item.strip() for item in slist]
                slist = [item for item in slist if item != '']
                return '\n'.join(slist)
            elif self.outputtype == 'compressed':
                slist = st.split('\n')
                slist = [item.strip() for item in slist]
                return ' '.join(slist)
        # else:
        st = ''
        dtls = self.model.dtls
        for objname in dtls:
            for obj in self.idfobjects[objname]:
                 st = st + obj.__repr__()
        return st
    def printidf(self):
        """print the idf"""
        print self.idfstr()
    def save(self):
        """save with comments"""
        s = self.idfstr()
        open(self.idfname, 'w').write(s)
    def saveas(self, filename):
        s = self.idfstr()
        open(filename, 'w').write(s)
    # def initread(self, idfname):
    #     """use the latest iddfile and read file fname"""
    #     from StringIO import StringIO
    #     from eppy.iddcurrent import iddcurrent
    #     iddfhandle = StringIO(iddcurrent.iddtxt)
    #     if self.getiddname() == None:
    #         self.setiddname(iddfhandle)
    #     self.idfname = idfname
    #     self.read()

class IDF3(IDF2):
    """subclass of IDF2. Uses functions of IDF1 and IDF2
    """
    def __init__(self, idfname=None):
        super(IDF3, self).__init__(idfname)
    def initread(self, idfname):
        """use the latest iddfile and read file fname
        idd is initialized only if it has not been done earlier"""
        from StringIO import StringIO
        from eppy.iddcurrent import iddcurrent
        iddfhandle = StringIO(iddcurrent.iddtxt)
        if self.getiddname() == None:
            self.setiddname(iddfhandle)
        self.idfname = idfname
        self.read()
    def initnew(self):
        """use the latest iddfile and opens a new file
        idd is initialized only if it has not been done earlier"""
        from StringIO import StringIO
        from eppy.iddcurrent import iddcurrent
        iddfhandle = StringIO(iddcurrent.iddtxt)
        if self.getiddname() == None:
            self.setiddname(iddfhandle)
        idfhandle = StringIO('')
        self.idfname = idfhandle
        self.read()
    def initreadtxt(self, idftxt):
        """use the latest iddfile and read txt
        idd is initialized only if it has not been done earlier"""
        from StringIO import StringIO
        from eppy.iddcurrent import iddcurrent
        iddfhandle = StringIO(iddcurrent.iddtxt)
        if self.getiddname() == None:
            self.setiddname(iddfhandle)
        idfhandle = StringIO(idftxt)
        self.idfname = idfhandle
        self.read()
        

IDF = IDF3
        
                                    
class something(IDF0):
    """docstring for something"""
    def __init__(self, arg):
        super(something, self).__init__()
        self.arg = arg
        



# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

"""helpers for pytest"""

# taken from python's unit test
# may be covered by Python's license 

def almostequal(first, second, places=7, printit=True):
    """docstring for almostequal"""
    if round(abs(second-first), places) != 0:
        if printit:
            print round(abs(second-first), places)
            print "notalmost: %s != %s" % (first, second)
        return False
    else:
        return True